# jahplayer - A widget library representation.

# Copyright (C) 2006 VM Inc.
# Released under the GPL.
# For more information, see http://www.jahshaka.org.

import platform
import os
import sys
import os.path
import re
import linecache
import random
import platform

import openpluginlib
import jahwidgets

class ostream:
	"C++ like ostream class. Use defines below e.g. cout << x << y << z << endl"
	def __init__(self, file):
		self.file = file

	def __lshift__(self, obj):
		self.file.write( str(obj) )
		return self

cout = ostream(sys.stdout)
cerr = ostream(sys.stderr)
endl = '\n'

# C printf like syntax, but with a newline
def printf( format, *args ): print format % args

# print a python object, useful with map etc
def printobj( obj ): print obj

def get_absolute_path( filename ):
	if platform.system( ) == "Darwin":
		# Try our bundle's Resources directory first
		path = jahwidgets.get_bundle_resource_path( filename )
		if os.access( path, os.F_OK ):
			return path
			
	elif platform.system() == "Windows":
		try:
			return os.path.join( os.environ[ "JAHTOOLSPATH" ], "jplayer", filename ).replace( '\\', '/' )
		except:
			pass

	path = sys.path[ 0 ]; 
	return os.path.join( os.path.abspath( path ), filename ).replace( '\\', '/' )

def get_absolute_reel( filename ):
	path = ""
	if platform.system( ) == "Windows":
		path = os.path.join( os.environ[ "APPDATA" ], "jah" )
	else:
		path = os.path.join( os.environ[ "HOME" ], ".jah" )
	
	if not os.access( path, os.F_OK ):
		os.mkdir( path, 0777 )

	return os.path.join( path, filename ).replace( '\\', '/' )

def get_files( path, pattern = None, regexp = None ):
	# Result to return
	result = []

	if regexp is None and pattern is not None:
		regexp = re.compile( pattern )
	elif regexp is None:
		regexp = re.compile( ".*" )

	try:
		names = os.listdir( path )

		for name in names:
			fullname = os.path.normpath( os.path.join( path, name ) ).replace( '\\', '/' )
			if regexp.match( name ):
				result.append( fullname )
			if os.path.isdir( fullname ):
				result += get_files( fullname, regexp = regexp )

		result.sort( )

	except os.error:
		pass

	return result

def bind_size_to( cw, ch, mw, mh ):
	ar = float( cw ) / float( ch )
	if mh * ar <= mw:
		return int( mh * ar ), int( mh )
	else:
		return int( mw ), int( mw / ar )

def traceit(frame, event, arg):
	if event == "line":
		lineno = frame.f_lineno
		filename = frame.f_globals["__file__"]
		if filename == "<stdin>":
			filename = "traceit.py"
		if filename.endswith(".pyc") or filename.endswith(".pyo"):
			filename = filename[:-1]
		name = frame.f_globals["__name__"]
		line = linecache.getline(filename, lineno)
		print "%s:%s: %s" % (name, lineno, line.rstrip())
	return traceit

class call_method_observer( openpluginlib.observer ):
	def __init__( self, method ):
		openpluginlib.observer.__init__( self )
		self.method = method

	def updated( self, subject ):
		self.method( subject )

class launch_dialog_observer( openpluginlib.observer ):
	def __init__( self, dialog ):
		openpluginlib.observer.__init__( self )
		self.dialog = dialog
		
	def updated( self, subject ):
		self.dialog.show( True )

def create_property_and_attach( handle, propname, observer ):
	prop = openpluginlib.property( openpluginlib.create_key_from_string( propname ) )
	prop.set_always_notify( True )
	jahwidgets.connect( propname, handle, prop )
	prop.attach( observer )
	return prop

def create_property( handle, propname ):
	prop = openpluginlib.property( openpluginlib.create_key_from_string( propname ) )
	jahwidgets.connect( propname, handle, prop )
	return prop

def create_property_and_set( propname, value, *parent_prop ):
	prop = openpluginlib.property( openpluginlib.create_key_from_string( propname ) )
	prop.set( value )
	if len( parent_prop ) and parent_prop[0].valid():
		parent_prop[0].append( prop )
		
	return prop
